@@ -2,29 +2,30 @@ package CXC::Number::Grid;
 
 # ABSTRACT: A class representing a one dimensional numeric grid
 
-use feature ':5.26';
-use Carp;
+use v5.26;
 
 use POSIX ();
 
-use Tree::Range::RB;
-
-use Types::Standard qw( ArrayRef InstanceOf Optional Enum Dict Bool );
-use Type::Params qw( compile );
-use Ref::Util qw[ is_plain_hashref is_blessed_ref ];
+use Types::Standard
+  qw( ArrayRef Bool Dict Enum HashRef InstanceOf Optional Slurpy );
+use Type::Params qw( signature signature_for );
+use Ref::Util    qw[ is_plain_hashref is_blessed_ref ];
 
 use CXC::Number::Grid::Types -types;
 use CXC::Number::Grid::Failure
   qw( parameter_interface parameter_constraint internal );
 
+use CXC::Number::Grid::Tree;
+
 use Safe::Isa;
 
 use Moo;
 
-use experimental 'signatures';
-use experimental 'refaliasing';
+our $VERSION = '0.09';
 
-our $VERSION = '0.08';
+use constant GridObject => InstanceOf [ ( __PACKAGE__ ) ];
+
+use experimental 'signatures';
 
 use Exporter::Shiny qw( join_n overlay_n );
 
@@ -32,10 +33,16 @@ use namespace::clean;
 
 use MooX::StrictConstructor;
 
-use overload '+' => \&_merge, fallback => 1, bool => sub { 1 };
+use overload
+  '!'      => \&_overload_not,
+  '|'      => \&_overload_or,
+  fallback => 1,
+  bool     => sub { 1 };
 
 my $DEBUG = $ENV{ __PACKAGE__ . "_DEBUG" } // 0;
 
+BEGIN { with 'MooX::Tag::TO_HASH' }    # so can see has
+
 sub _convert ( $self, $bignum ) {
     require Ref::Util;
 
@@ -56,6 +63,7 @@ has oob => (
     is      => 'ro',
     isa     => Bool,
     default => 0,
+    to_hash => 1,
 );
 
 
@@ -65,11 +73,12 @@ has oob => (
 
 
 has _raw_edges => (
-    is       => 'lazy',
+    is       => 'ro',
     init_arg => 'edges',
     isa      => BinEdges,
     required => 1,
     coerce   => 1,
+    to_hash  => 'edges',
 );
 
 
@@ -83,6 +92,7 @@ has _include => (
     init_arg => 'include',
     isa      => ArrayRef [ Enum [ 0, 1 ] ],
     builder  => sub { [ ( 1 ) x $_[0]->nbins ] },
+    to_hash  => 'include,if_exists',
 );
 
 
@@ -341,64 +351,6 @@ sub overlay ( $self, @args ) {
 }
 
 
-package CXC::Number::Grid::Range {
-    use Moo;
-    use experimental 'signatures';
-    use experimental 'declared_refs';
-    use experimental 'refaliasing';
-
-    use overload
-      fallback => 0,
-      bool     => sub { 1 },
-      '""'     => \&to_string,
-      '.'      => \&concatenate;
-
-    has layer   => ( is => 'ro' );
-    has include => ( is => 'ro' );
-    has lb      => ( is => 'ro' );
-    has ub      => ( is => 'ro' );
-
-    around BUILDARGS => sub ( $orig, $class, @args ) {
-
-        my \%args = ref $args[0] ? $args[0] : {@args};
-
-        @args{ 'layer', 'include' } = delete( $args{value} )->@*
-          if defined $args{value};
-
-        return $class->$orig( \%args );
-    };
-
-    sub to_string ( $self, $ =, $ = ) {
-        my $ub      = $self->ub      // 'undef';
-        my $lb      = $self->lb      // 'undef';
-        my $layer   = $self->layer   // 'undef';
-        my $include = $self->include // 'undef';
-        "( $lb, $ub ) => { layer => $layer, include => $include }";
-    }
-
-    sub concatenate ( $self, $other, $swap ) {
-        my $str = $self->to_string;
-        return $swap ? $other . $str : $str . $other;
-    }
-
-    sub value ( $self ) {
-        return [ $self->layer, $self->include ];
-    }
-
-}
-
-sub _dump_tree ( $tree ) {
-    my $ic = $tree->range_iter_closure;
-    say "------------";
-    while ( my ( $v, $lb, $ub ) = $ic->() ) {
-        $v = defined $v ? "[ @{[ join( ', ', $v->@*) ]} ]" : 'undef';
-        $lb //= 'undef';
-        $ub //= 'undef';
-        say "( $lb, $ub )\t=> $v";
-    }
-    say "------------";
-}
-
 
 
 
@@ -482,23 +434,26 @@ sub _dump_tree ( $tree ) {
 
 sub overlay_n {
 
-    state $check = compile(
-        ArrayRef [ InstanceOf [ ( __PACKAGE__ ) ] ],
-        Optional [
-            Dict [
-                snap_dist => Optional [BigPositiveOrZeroNum],
-                snap_to   => Optional [ Enum [ 'underlay', 'overlay' ] ],
+    state $signature = signature(
+        positional => [
+            ArrayRef [GridObject],
+            Optional [
+                Dict [
+                    snap_dist => Optional [BigPositiveOrZeroNum],
+                    snap_to   => Optional [ Enum [ 'underlay', 'overlay' ] ],
+                ],
             ],
         ],
     );
 
     my @dict = ( @_ && is_plain_hashref( $_[-1] ) ? pop @_ : () );
 
-    my ( $grids, $opt ) = $check->( \@_, @dict, );
+    my ( $grids, $opt ) = $signature->( \@_, @dict, );
+
     $opt->{snap_to}   //= 'underlay';
     $opt->{snap_dist} //= Math::BigFloat->bzero;
 
-    my $tr = Tree::Range::RB->new( { cmp => sub { $_[0] <=> $_[1] } } );
+    my $tr = CXC::Number::Grid::Tree->new;
 
     my $gi = 0;
     for my $grid ( $grids->@* ) {
@@ -512,115 +467,13 @@ sub overlay_n {
 
         # snap bin edges if they are from different grids and are too close.
         # do this in the loop so that there are only two grids at a time
-        _snap_to( $tr, $gi, $opt->{snap_to}, $opt->{snap_dist} )
+        $tr->snap_overlaid( $gi, $opt->{snap_to}, $opt->{snap_dist} )
           if $gi > 1;
     }
 
-
-    my @edges;
-    my @include;
-    my $ic = $tr->range_iter_closure;
-
-    my ( $v, $lower, $upper ) = $ic->();
-    push @edges, $upper;
-
-    while ( ( $v, $lower, $upper ) = $ic->() ) {
-        if ( defined $upper ) {
-            push @edges, $upper;
-            push @include, ( $v // [0] )->[-1];
-        }
-    }
-
-    return __PACKAGE__->new( edges => \@edges, include => \@include );
+    return $tr->to_grid;
 }
 
-sub _snap_to ( $tr, $layer, $snap_to, $snap_dist ) {
-
-    return if $snap_dist == 0;
-
-    # Tree::Range doesn't represent a range as a node with the ability
-    # to visit a predecessor.  It essentially only allows one way
-    # tree traversal, so we need to traverse it forwards to handle
-    # snapping to the right, and backwards to handle snapping to the left.
-    _merge_bins( $tr, $layer, $snap_dist, $snap_to, $_ ) for qw( right left );
-}
-
-sub _merge_bins ( $tr, $layer, $snap_dist, $snap_to, $scan_direction ) {
-
-    defined( my $scan_reversed = { right => 0, left => 1 }->{$scan_direction} )
-      // die( "illegal scan direction: '$scan_direction'" );
-
-    my sub iter ( $key = undef ) {
-        my $iter = $tr->range_iter_closure( $key, $scan_reversed );
-        # first range goes from +-inf to real bound; remove
-        $iter->() unless defined $key;
-        return $iter;
-    }
-
-    my $iter = iter();
-
-    my sub next_range {
-        my @r = $iter->();
-        return @r
-          ? CXC::Number::Grid::Range->new( {
-                value => $r[0],
-                lb    => $r[1],
-                ub    => $r[2],
-            } )
-          : undef;
-    }
-
-    my $current = next_range();
-    my $next;
-
-    my %SnapTo = (
-        overlay => {
-            right => sub {
-                # would prefer
-                # $tr->range_set( $prev->lb, $current->ub, $current->value );
-                # but there's no way to get $prev from Tree::Range.
-                #
-                # This code depends upon Tree::Range storing [ $lb, $value ]
-                # in each node, so deleting a node extends the previous
-                # range.
-                $tr->delete( $current->lb );
-                $iter = iter( $current->lb );
-            },
-            left => sub {
-                $tr->range_set( $next->lb, $current->ub, $next->value );
-                $iter = iter( $current->ub );
-            }
-        },
-        underlay => {
-            right => sub {
-                $tr->range_set( $current->lb, $next->ub, $next->value );
-                $iter = iter( $current->lb );
-
-            },
-            left => sub {
-                $tr->range_set( $next->lb, $current->ub, $next->value );
-                $iter = iter( $current->ub );
-            },
-        },
-    );
-
-    my $snap = $SnapTo{$snap_to}{$scan_direction};
-
-    while ( defined( $next = next_range() ) ) {
-
-        if (   abs( $current->ub - $current->lb ) <= $snap_dist
-            && $next->layer == $layer
-            && $current->layer < $next->layer )
-        {
-            $snap->();
-            # all of the $snap routines reset the iterator
-            $current = next_range();
-        }
-        else {
-            $current = $next;
-        }
-    }
-}
 
 
 
@@ -848,19 +701,21 @@ sub join_n {
     );
 
 
-    state $check = compile(
-        ArrayRef [ InstanceOf [ ( __PACKAGE__ ) ] ],
-        Optional [
-            Dict [
-                gap => Enum [ qw(
-                      shift-right
-                      shift-left
-                      snap-right
-                      snap-left
-                      snap-both
-                      include
-                      exclude
-                    )
+    state $check = signature(
+        positional => [
+            ArrayRef [GridObject],
+            Optional [
+                Dict [
+                    gap => Enum [ qw(
+                          shift-right
+                          shift-left
+                          snap-right
+                          snap-left
+                          snap-both
+                          include
+                          exclude
+                        )
+                    ],
                 ],
             ],
         ],
@@ -936,6 +791,152 @@ sub join_n {
 
 
 
+sub not ( $self ) {
+
+    my $hash = $self->TO_HASH;
+    $_ = $_ ? 0 : 1 for $hash->{include}->@*;
+    return __PACKAGE__->new( $hash );
+}
+
+
+
+
+
+sub _overload_not ( $self, $, $ ) {
+    return $self->not;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+sub _overload_or ( $self, $other, $ ) {
+
+    $other->$_isa( __PACKAGE__ )
+      or die(
+        "can only perform the | operation between two ${ \__PACKAGE__ } objects "
+      );
+    return $self->or( $other );
+}
+
+
+
+
+
+
+
+
+
+
+sub or ( $self, @args ) {
+    return or_n( $self, @args );
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+signature_for or_n => ( positional => [ Slurpy [ ArrayRef [GridObject] ] ], );
+sub or_n ( $grids ) {
+    my $tr = CXC::Number::Grid::Tree->new;
+
+    my $gi = 0;
+
+    for my $grid ( $grids->@* ) {
+        ++$gi;
+        my $edges   = $grid->_raw_edges;
+        my $include = $grid->include;
+
+        my @idx = 0 .. ( $grid->nbins - 1 );
+        @idx = grep $include->[$_], @idx
+          if $gi > 1;
+
+        $tr->range_set( $edges->@[ $_, $_ + 1 ], $include->[$_] ) for @idx;
+    }
+
+    return $tr->to_grid;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+sub combine_bins ( $self ) {
+
+    my $edges   = $self->_raw_edges;
+    my $include = $self->include;
+
+    my $tr = CXC::Number::Grid::Tree->new( {
+        'equal-p' =>
+          sub { defined $_[0] && defined $_[1] && $_[0] == $_[1] },
+    } );
+
+    $tr->range_set( $edges->[$_], $edges->[ $_ + 1 ], $include->[$_] )
+      for 0 .. ( $include->@* - 1 );
+
+    return $tr->to_grid;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -972,6 +973,21 @@ sub pdl ( $self ) {
 
 
 
+
+
+
+
+
+
+
+
+sub _modify_hashr ( $self, $hash ) {
+    $hash->{edges}   = [ map { $_->copy } $hash->{edges}->@* ];
+    $hash->{include} = [ $hash->{include}->@* ]
+      if exists $hash->{include};
+}
+
+
 1;
 
 #
@@ -988,8 +1004,8 @@ __END__
 
 =pod
 
-=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory overlayed nbins nedges
-oob ub unobscured Extrema bignum pdl
+=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory nbins nedges oob ub
+unobscured Extrema bignum pdl
 
 =head1 NAME
 
@@ -997,7 +1013,7 @@ CXC::Number::Grid - A class representing a one dimensional numeric grid
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
@@ -1160,6 +1176,35 @@ Overlay one or more grids on top of C<$grid1> and return a new grid.
 
 See C<< L</overlay_n> >> for a description of the options.
 
+=head2 not
+
+  $flipped = $grid->not;
+
+return a copy of C<$grid> with a Boolean not of its include values.
+
+=head2 or
+
+   $ABC = $A->or( $B, $C, ..., ?\%options );
+
+Perform the logical or of grids and return a new grid.
+See C<< L</or_n> >> for a description of the options.
+
+=head2 combine_bins
+
+  $combined = $grid->combine_bins
+
+Combine adjacent bins with the same C<include> value.
+
+For instance, a grid with the following construction:
+
+    edges   => [ 0, 2, 4, 8, 12, 16 ]
+    include => [ 0, 0, 1, 1, 0 ]
+
+Would be combined into
+
+    edges   => [ 0, 4, 12, 16 ]
+    include => [ 0, 1, 0 ]
+
 =head2 bignum
 
   $bin_edges = $grid->bignum->bin_edges;
@@ -1187,6 +1232,22 @@ Returns an object which returns piddles for the following methods
   lb        -> piddle
   ub        -> piddle
 
+=head1 OVERLOAD
+
+=head2 !
+
+The logical NOT C<!> operator is overloaded; see L</not> for details.
+
+=head2 |
+
+   $AB = $A | $B
+
+The logical OR C<!> operator is overloaded via
+
+  $AB = $A->or($B);
+
+see L</or> for details.
+
 =head1 SUBROUTINES
 
 =head2 overlay_n
@@ -1385,8 +1446,45 @@ Add a new bin, and mark it as being excluded
 
 =back
 
+=head2 or_n
+
+  $grid = CXC::Number::Grid::or_n( $grid1, $grid2, ..., $gridN, ?\%options );
+
+Logical or of grids based upon their include values. For example, given
+two grids:
+
+  Grid A:
+
+    edges   => [ 0, 2, 4, 8, 12, 16 ]
+    include =>   [ 0, 1, 0, 1,  0 ]
+
+  Grid B;
+    edges   => [ 0, 3, 6, 9, 10, 11, 16 ]
+    include =>   [ 1, 0, 1, 0,  1,  0 ]
+
+The result of
+
+     $A | $B
+
+would be
+
+     edges   => [ 0, 3, 4, 6, 9, 10, 11, 12, 16 ];
+     include =>   [ 1, 1, 0, 1, 1,  1,  1,  0 ];
+
 =head1 INTERNALS
 
+=head2 Methods
+
+=head3 _overload_not
+
+=head3 _modify_hashr
+
+This is called by MooX::Tag::TO_HASH to modify the generated hash
+representation.
+
+This routine makes copies of the structures so that the hash
+can be modified without affecting the parent object.
+
 =for Pod::Coverage BUILD
 
 =head1 SUPPORT
