@@ -2,7 +2,7 @@ package CXC::Number::Sequence;
 
 # ABSTRACT: Numerical Sequence Generation
 
-use feature ':5.24';
+use v5.26;
 
 use Carp;
 
@@ -16,7 +16,7 @@ use Moo;
 
 use experimental 'signatures';
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use namespace::clean;
 
@@ -195,7 +195,7 @@ CXC::Number::Sequence - Numerical Sequence Generation
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
