@@ -5,10 +5,7 @@ package CXC::Number::Sequence::Linear;
 use strict;
 use warnings;
 
-use feature ':5.24';
-use experimental 'lexical_subs';
-
-use Carp;
+use v5.26;
 
 use Hash::Wrap 0.11 {
     -as        => 'wrap_attrs_ro',
@@ -26,7 +23,7 @@ use enum
 
 use Moo;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 extends 'CXC::Number::Sequence';
 
@@ -408,7 +405,7 @@ CXC::Number::Sequence::Linear - Numeric Sequence with Equal Spacing
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
