@@ -5,14 +5,11 @@ package CXC::Number::Sequence::Ratio;
 use strict;
 use warnings;
 
-use feature ':5.24';
-use experimental 'lexical_subs';
+use v5.26;
 
 # ABSTRACT: ratio sequence
 
-use Carp;
-
-use Types::Standard qw( Optional );
+use Types::Standard        qw( Optional );
 use Types::Common::Numeric qw( PositiveInt );
 use Math::BigInt;
 use List::Util qw( max );
@@ -41,7 +38,7 @@ extends 'CXC::Number::Sequence';
 
 use namespace::clean;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 my sub nelem {
     my ( $ratio, $w0, $Dr ) = @_;
@@ -331,7 +328,7 @@ CXC::Number::Sequence::Ratio - Numeric Sequence with Relative Fractional Spacing
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
