@@ -2,11 +2,11 @@ package CXC::Number::Sequence::Role::BigNum;
 
 # ABSTRACT: Role to return Math::BigFloats from Sequences
 
-use feature ':5.24';
+use v5.26;
 
 use Moo::Role;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use experimental 'signatures';
 use namespace::clean;
@@ -43,7 +43,7 @@ CXC::Number::Sequence::Role::BigNum - Role to return Math::BigFloats from Sequen
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
