@@ -2,7 +2,7 @@ package CXC::Number::Sequence::Role::PDL;
 
 # ABSTRACT: Role to return PDL objects from Sequences
 
-use feature ':5.24';
+use v5.26;
 use PDL::Lite ();
 
 use Moo::Role;
@@ -10,7 +10,7 @@ use Moo::Role;
 use experimental 'signatures';
 use namespace::clean;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 sub _convert ( $self, $bignum ) {
     require Ref::Util;
@@ -44,7 +44,7 @@ CXC::Number::Sequence::Role::PDL - Role to return PDL objects from Sequences
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
