@@ -4,17 +4,15 @@ package CXC::Number::Sequence::Utils;
 use strict;
 use warnings;
 
-use feature ':5.24';
+use v5.26;
 use experimental 'signatures';
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 # ABSTRACT: sequence utilities
 
-use feature 'state';
-
 use Exporter::Shiny qw( buildargs_factory load_class );
-use Type::Params qw( compile_named compile_named_oo );
+use Type::Params    qw( compile_named compile_named_oo );
 use Types::Standard -types;
 use Types::Common::Numeric qw( PositiveInt );
 
@@ -230,7 +228,7 @@ CXC::Number::Sequence::Utils - Utilities for CXC::Number::Sequence generators
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
